//
//  DataManager.swift
//  Reminderz
//
//  Created by Cory Bohon on 4/16/18.
//  Copyright © 2018 MartianCraft. All rights reserved.
//

import Cocoa

class DataManager: NSObject {

    static let shared = DataManager()
    
    var storedTodos = [TodoItem]()
    
    override init() {
        super.init()
        
        if let data = UserDefaults.standard.object(forKey: "todoItems") as? Data {
            storedTodos = NSKeyedUnarchiver.unarchiveObject(with: data) as! Array<TodoItem>
            self.save()
        }
    }
    
    func add(item: TodoItem) {
        storedTodos.append(item)
        save()
        createNotification(name: item.todoName, date: item.todoDueDate)
    }
    
    func remove(item: TodoItem) {
        if let index = storedTodos.index(of: item) {
            storedTodos.remove(at: index)
            save()
            removeNotification(name: item.todoName, date: item.todoDueDate)
        }
    }
    
    func save() {
        storedTodos.sort { (todo1: TodoItem, todo2: TodoItem) -> Bool in
            if todo1.todoDueDate.compare(todo2.todoDueDate) == .orderedDescending {
                return true
            }
            return false
        }
        
        let data = NSKeyedArchiver.archivedData(withRootObject: storedTodos)
        UserDefaults.standard.set(data, forKey: "todoItems")
        NotificationCenter.default.post(name: NSNotification.Name("dataChanged"), object: nil)
    }
    
    func createNotification(name: String, date: Date) {
        let notification: NSUserNotification = NSUserNotification()
        notification.deliveryDate = date
        notification.title = "Reminderz"
        notification.subtitle = name
        NSUserNotificationCenter.default.scheduleNotification(notification)
    }
    
    func removeNotification(name: String, date: Date) {
        for notification in NSUserNotificationCenter.default.scheduledNotifications {
            if notification.subtitle! == name && notification.deliveryDate?.compare(date) == .orderedSame {
                NSUserNotificationCenter.default.removeScheduledNotification(notification)
            }
        }
    }
    
    func removeOldNotifications() {
        NSUserNotificationCenter.default.removeAllDeliveredNotifications()
    }

}
