//
//  ViewController.swift
//  Reminderz
//
//  Created by Cory Bohon on 4/16/18.
//  Copyright © 2018 MartianCraft. All rights reserved.
//

import Cocoa

class ViewController: NSViewController, NSTableViewDelegate, NSTableViewDataSource {
    
    @IBOutlet var tableView: NSTableView!
    @IBOutlet var todoNameField: NSTextField!
    @IBOutlet var todoDueDatePicker: NSDatePicker!

    let dateFormatter: DateFormatter = DateFormatter()
    
    override func viewDidLoad() {
        super.viewDidLoad()

        todoDueDatePicker.dateValue = Date()
        todoNameField.becomeFirstResponder()
        
        dateFormatter.dateStyle = .short
        dateFormatter.timeStyle = .short
        
        tableView.delegate = self
        tableView.dataSource = self
        
        NotificationCenter.default.addObserver(self, selector: #selector(updateData(sender:)), name: NSNotification.Name("dataChanged"), object: nil)
    }

    @IBAction func saveButtonClicked(_: Any) {
        if todoNameField.stringValue.count > 0 {
            let todo = TodoItem(name: todoNameField.stringValue, dueDate: todoDueDatePicker.dateValue)
            DataManager.shared.add(item: todo)
            
            todoNameField.stringValue = ""
            todoDueDatePicker.dateValue = Date()
        } else {
            let alertView = NSAlert()
            alertView.alertStyle = .informational
            alertView.messageText = "Todo name is invalid"
            alertView.informativeText = "The name field must have a value before adding the todo to the list."
            alertView.runModal()
        }
    }
    
    // MARK: TableViewDataSource and TableViewDelegate
    func numberOfRows(in tableView: NSTableView) -> Int {
        return DataManager.shared.storedTodos.count
    }
    
    func tableView(_ tableView: NSTableView, viewFor tableColumn: NSTableColumn?, row: Int) -> NSView? {
        let todo = DataManager.shared.storedTodos[row]
        
        var cellValue = ""
        var cellIdentifier = NSUserInterfaceItemIdentifier("")
        
        if tableColumn?.identifier.rawValue == "itemColumn" {
            cellValue = todo.todoName
            cellIdentifier = NSUserInterfaceItemIdentifier("nameCell")
        } else if tableColumn?.identifier.rawValue == "dueDateColumn" {
            cellValue = self.dateFormatter.string(from: todo.todoDueDate)
            cellIdentifier = NSUserInterfaceItemIdentifier("dateCell")
        }
        
        if let cell = tableView.makeView(withIdentifier: cellIdentifier, owner: nil) as? NSTableCellView {
            
            cell.textField?.stringValue = cellValue
            
            if todo.todoDueDate.compare(Date()) == .orderedAscending {
                cell.alphaValue = 0.25
            }

            return cell
        }
        
        return nil
    }
    
    @objc func updateData(sender: NSNotification) {
        tableView.reloadData()
    }
    
    // MARK: Keyboard Shortcuts
    override func keyDown(with event: NSEvent) {
        interpretKeyEvents([event])
    }
    
    override func deleteBackward(_ sender: Any?) {
        let selectedRow: Int = tableView.selectedRow
        if selectedRow == -1 {
            return
        }
        
        DataManager.shared.remove(item: DataManager.shared.storedTodos[selectedRow])
    }
}

